@@ -651,6 +651,7 @@ static int omap_hsmmc_execute_tuning(struct udevice *dev, uint opcode)
 		printf("Couldn't get temperature for tuning\n");
 		return ret;
 	}
+	temperature /= 1000;
 	val = readl(&mmc_base->dll);
 	val |= DLL_SWT;
 	writel(val, &mmc_base->dll);
@@ -780,6 +781,14 @@ tuning_error:
 	return ret;
 }
 #endif
+
+static void omap_hsmmc_send_init_stream(struct udevice *dev)
+{
+	struct omap_hsmmc_data *priv = dev_get_priv(dev);
+	struct hsmmc *mmc_base = priv->base_addr;
+
+	mmc_init_stream(mmc_base);
+}
 #endif
 
 static void mmc_enable_irq(struct mmc *mmc, struct mmc_cmd *cmd)
@@ -1515,9 +1524,10 @@ static const struct dm_mmc_ops omap_hsmmc_ops = {
 	.get_wp		= omap_hsmmc_getwp,
 #endif
 #if CONFIG_IS_ENABLED(MMC_SUPPORTS_TUNING)
-	.execute_tuning = omap_hsmmc_execute_tuning,
+	.execute_tuning    = omap_hsmmc_execute_tuning,
 #endif
-	.wait_dat0	= omap_hsmmc_wait_dat0,
+	.send_init_stream  = omap_hsmmc_send_init_stream,
+	.wait_dat0	   = omap_hsmmc_wait_dat0,
 };
 #else
 static const struct mmc_ops omap_hsmmc_ops = {
