@@ -11,10 +11,8 @@
 #define PHYS_SDRAM_1_SIZE		0x40000000	/* 1GiB on VINING_FPGA */
 
 /* Booting Linux */
-#define CONFIG_SYS_BOOTM_LEN	0x2000000	/* 32 MiB */
 
 /* Extra Environment */
-#define CONFIG_HOSTNAME			"socfpga_vining_fpga"
 
 /*
  * Active LOW GPIO buttons:
@@ -29,7 +27,7 @@
  *                            Linux system after 5 seconds
  */
 
-#define CONFIG_EXTRA_ENV_SETTINGS \
+#define CFG_EXTRA_ENV_SETTINGS \
 	"verify=n\0" \
 	"consdev=ttyS0\0"						\
 	"baudrate=115200\0"						\
@@ -42,7 +40,6 @@
 	"hostname=vining_fpga\0"					\
 	"kernel_addr_r=0x10000000\0"					\
 	"fdt_addr_r=0x20000000\0"					\
-	"fdt_high=0xffffffff\0"						\
 	"initrd_high=0xffffffff\0"					\
 	"dfu_alt_info=qspi0 sf 0:0;qspi1 sf 0:1\0"			\
 	"mtdparts_0_16m=ff705000.spi.0:" /* 16MiB+128MiB SF config */	\
